# Hash.pm
#
# Author: Liraz Siri <liraz@hushmail.com>
#
# Copyright (c) 1999 Liraz siri <liraz@hushmail.com>, Ariel, Israel,
#               All rights reserved.
#
# Created: Fri Apr 23 05:35:14 IDT 1999
#
# VRS Hash type tie magic.

package Tie::VRS::Hash;

use strict;

use Tie::VRS::Base;
use vars qw(@ISA);
@ISA = qw(Tie::VRS::Base);

sub init {
    my ($self, $hash) = @_;
    while(my ($key, $var) = each %$hash) {
        $self->STORE($key, $var);
    }
}

*new = \&TIEHASH;
sub TIEHASH {
    my ($class, @args) = @_;
    my $self = bless {}, $class;
    $self->type('HASH');
    $self->init(@args);
    return $self;
}

sub fetch {
    my ($self, $key) = @_;
    return $self->ref->{$key};
}

# array fetch and hash fetch are identical, any room for abstraction?
sub FETCH {
    my ($self, $key) = @_;
    return($self->enchant_ref_cached($key) ||
           $self->enchant_ref($self->fetch($key), $key));
}

sub store {
    my ($self, $key, $var) = @_;
    my $ref = $self->ref;
    $ref->{$key} = $var;
    # TODO: remove any replaced magic references.
    $self->changed;
    return $var;
}

sub STORE {
    my ($self, $key, $var) = @_;
    $self->enchant_unref($key);
    $self->store($key, $var);
    $self->enchant_ref($var, $key);
    return $var;
}

sub FIRSTKEY {
    my $self = shift;
    my $ref = $self->ref;
    my $keys = keys %$ref;
    $self->start_of_iteration if $keys;
    return each %$ref;
}

sub NEXTKEY {
    my $self = shift;
    my $ref = $self->ref;
    my $key = each %$ref;
    $self->end_of_iteration unless $key;
    return $key;
}

sub EXISTS {
    my ($self, $key) = @_;
    my $ref = $self->ref;
    return exists $ref->{$key};
}

sub DELETE {
    my ($self, $key) = @_;
    my $ref = $self->ref;
    $self->enchant_unref($key);
    my $return = delete $ref->{$key};
    $self->changed;
    return $return;
}

sub CLEAR {
    my $self = shift;
    my $ref = $self->ref;
    $self->enchant_unref_all;
    my @return = %$ref = ();
    $self->changed;
    return @return;
}

1;
