# VRS.pm
#
# Author: Liraz Siri <liraz@hushmail.com>
#
# Copyright (c) 1999 Liraz Siri <liraz@hushmail.com>, Ariel, Israel,
#               All rights reserved.
#
# Created: Thu Apr 15 23:54:36 IDT 1999
#
# VRS (Virtual Reference System) perl tie API.

package Tie::VRS;
use strict;

sub TIESCALAR {
    my $class = shift;

    use Tie::VRS::Scalar;
    return Tie::VRS::Scalar->new(@_);
}

sub TIEARRAY {
    my $class = shift;

    use Tie::VRS::Array;
    return Tie::VRS::Array->new(@_);
}

sub TIEHASH {
    my $class = shift;

    use Tie::VRS::Hash;
    return Tie::VRS::Hash->new(@_);
}

1;

__END__

=head1 NAME

Tie::VRS module - Tie API to the perl Virtual Reference System.

=head1 SYNOPSIS

use Tie::VRS;

tie $scalar => Tie::VRS, $scalar;
tie @array => Tie::VRS, @array;
tie %hash => Tie::VRS, %hash;

=head1 DESCRIPTION

The VRS is an implementation of a pathological virtual reference system
which tries to simulate the way perl references normally behave, 
under twilight-zone circumstances, in a really complex way.

VRS is a byproduct of the IPC::UniShare module goal, to implement
a functional transparent interface to shared memory. I found introducing
references across a dynamicly shared-memory structure is very difficult
to implement correctly. It sounds simple, but it turns out to be extremely 
tricky, which might explain why it had never been done before.

Briefly, VRS is an evil, obscure, wave-a-dead-chicken at 
example of perl tie voodoo. It works, but I'm not always sure I really
understand why. Be careful hacking on the code, it is tightly packed
and very interdependant. Just make sure you really understand what
everything does before you go around changing things.

You've been warned.

Why bother? Because it should allow you to get away with all sorts of
subtle obscure wizadry. Tricking data and objects that they're running
on local memory, while they're actually accessing memory
from the beyond. (beyond the process, across the network, ...)

Also, if you're planning to jump through any sort of mental hoops
with perl tie magic, VRS is good reference material.

=head1 INTERFACE

# tie a scalar to VRS.
tie $scalar, Tie::VRS, $scalar;

# tie an array to VRS.
tie @array, Tie::VRS, @array;

# tie a hash to VRS.
tie %hash, Tie::VRS, %hash;

=head1 INTERNALS

I put alot of thought into making the source code as readable as
possible. It's OO, simple, consistent and clean.

If you have any questions, read it.

=head1 AUTHOR

 Author: Liraz Siri <liraz@hushmail.com>

 Copyright (c) 1999 Liraz Siri <liraz@hushmail.com>, Ariel, Israel
                    All rights reserved.

 You may redistribute VRS *only* under the terms of the GNU public
 license.
