# VRS/Array.pm
#
# Author: Liraz Siri <liraz@hushmail.com>
#
# Copyright (c) 1999 Liraz siri <liraz@hushmail.com>, Ariel, Israel,
#               All rights reserved.
#
# Created: Fri Apr 23 05:35:14 IDT 1999
#
# VRS array type tie magic.

package Tie::VRS::Array;
use strict;
use Tie::VRS::Base;

use vars qw(@ISA);
@ISA = qw(Tie::VRS::Base);

use Debug;

sub init {
    my ($self, $array) = @_;
    if(@$array) {
        $self->CLEAR();
        foreach (@$array) { $self->PUSH($_) }
    }
}

*new = \&TIEARRAY;
sub TIEARRAY {
    my ($class, @args) = @_;
    my $self = bless {}, $class;
    $self->type('ARRAY');
    $self->init(@args);
    return $self;
}

sub EXTEND {
    my $self = shift;
    return $self->STORESIZE(@_);
}

sub fetch {
    my ($self, $index) = @_;
    my $ref = $self->ref;
    return $ref->[$index];
}

sub FETCH {
    my ($self, $index) = @_;
    return($self->enchant_ref_cached($index) ||
           $self->enchant_ref($self->fetch($index), $index));
}

sub store {
    my ($self, $index, $var) = @_;
    my $ref = $self->ref;
    $ref->[$index] = $var;
    $self->changed;
    return $var;
}

sub STORE {
    my ($self, $index, $var) = @_;
    $self->enchant_unref($index);
    $self->store($index, $var);
    $self->enchant_ref($var, $index);
    return $var;
}

sub FETCHSIZE {
    my $self = shift;
    my $ref = $self->ref;
    return scalar @$ref;
}

sub STORESIZE {
    my ($self, $size) = @_;
    my $ref = $self->ref;

    if($#{$ref} > $size) {
        for (0 .. ($#{$ref} - $size)) {
            $self->enchant_unref($#{$ref} - $_);
        }
    }
    $#{$ref} = ($size - 1);

    $self->changed;
    return $size;
}

sub CLEAR {
    my $self = shift;
    my $ref = $self->ref;

    $self->enchant_unref_all;
    @$ref = ();

    $self->changed;
    return ();
}

sub POP {
    my $self = shift;
    my $ref = $self->ref;

    $self->enchant_unref($#{$ref});
    my $var = pop @$ref;

    $self->changed;
    return $var;
}

sub PUSH {
    my ($self, @vars) = @_;
    my $ref = $self->ref;

    my $size = scalar @$ref;
    for (0 .. $#vars) {
        $ref->[$size + $_] = $vars[$_];
        $self->enchant_ref($vars[$_], $size + $_);
    }
    
    $self->changed;
    return ($size + @vars);
}

sub SHIFT {
    my $self = shift;
    my $ref = $self->ref;
    $self->enchant_unref(0);
    $self->enchant_ref_array_shift(-1, 0);
    my $var = shift @$ref;
    $self->changed;
    return $var;
}

sub UNSHIFT {
    my ($self, @args) = @_;
    my $ref = $self->ref;
    $self->enchant_ref_array_shift(scalar @args, 0);
    my $return = unshift @$ref, @args;
    for (0 .. $#args) {
        $self->enchant_ref($args[$_], $_);
    }
    $self->changed;

    return $return;
}

sub SPLICE {
    Debug->called(@_);
    my ($self, $offset, $length, @list) = @_;
    my $ref = $self->ref;

    for ($offset .. ($offset + $length - 1)) {
        $self->enchant_unref($_);
    }
    $self->enchant_ref_array_shift(@list - $length, $offset);
    my @return = splice @$ref, $offset, $length, @list;

    for (0 .. $#list) {
        $self->enchant_ref($list[$_], $_ + $offset);
    }
    $self->changed;
    return @return;
}

### look it's a kludge!
*tied_ref = \&Tie::VRS::Base::tied_ref;

### shift our reference array corrosponding to a change in our
### array data type (which may be a shift, an unshift or a splice)
###
sub enchant_ref_array_shift {
    Debug->called(@_);
    my ($self, $shift, $offset) = @_;
    return unless $shift != 0 && $self->type eq 'ARRAY';

    my $cache = $self->enchant_ref_cache;

    ### contact our references, change (from their perspective)
    ### their position in our array.
    ###
    for my $index ($offset .. $#{$cache}) {
        my $ref = $cache->[$index];
        (tied_ref($ref))->chkey_parent($self, $index => ($index + $shift))
            if $ref;
    }
    if($shift < 0) {
        splice @$cache, $offset, -$shift;
    } else {
        ### inject undefed cells into our reference cache.
        ###
        my @undef_list;
        $#undef_list = ($shift - 1) if($shift > 0);
        splice @$cache, $offset, 0, @undef_list;
    }
}

1;
